@@ -29,16 +29,17 @@ class CanonicalException(Exception):
     #: The default log message. String formatting is applied.
     log_message = "Caught fatal {code} (id: {id})"
 
-    def __init__(self, code=None, message=None, detail=None, hint=None,
-        http_status_code=None):
+    def __init__(self, id=None, code=None, message=None, detail=None, hint=None,
+        http_status_code=None, **params):
         """Initialize a new :class:`CanonicalException`."""
-        self.id = uuid.uuid4()
+        self.id = id or uuid.uuid4()
         self.code = code or self.code
         self.message = message or self.message
         self.detail = detail or self.detail
         self.hint = hint or self.hint
         self.http_status_code = http_status_code or self.http_status_code
         self.schema = self.schema_class()
+        self.params = params
 
     def as_dict(self):
         """Return a dictionary containing the exception properties."""
@@ -47,3 +48,9 @@ class CanonicalException(Exception):
     def log(self, func, message=None):
         """Use `logger` to log the exception as `level`."""
         return func((message or self.log_message).format(**self.as_dict()))
+
+    def __str__(self) -> str:
+        return repr(self)
+
+    def __repr__(self) -> str:
+        return f'<{type(self).__name__}: {self.code} (id: {self.id})>'
